@@ -32,6 +32,7 @@ use Behat\Behat\Context\Step\Given as Given,
     Behat\Behat\Context\Step\When as When,
     Behat\Behat\Context\Step\Then as Then,
     Behat\Gherkin\Node\TableNode as TableNode,
+    Behat\Gherkin\Node\PyStringNode as PyStringNode,
     Behat\Mink\Element\NodeElement as NodeElement,
     Behat\Mink\Exception\ExpectationException as ExpectationException,
     Behat\Mink\Exception\ElementNotFoundException as ElementNotFoundException;
@@ -63,9 +64,6 @@ class behat_forms extends behat_base {
     /**
      * Fills a form with field/value data. More info in http://docs.moodle.org/dev/Acceptance_testing#Providing_values_to_steps.
      *
-     * Backport of Moodle 2.7 method to help backporting
-     * features to 2.6.
-     *
      * @Given /^I set the following fields to these values:$/
      * @throws ElementNotFoundException Thrown by behat_base::find
      * @param TableNode $data
@@ -83,34 +81,6 @@ class behat_forms extends behat_base {
         }
     }
 
-    /**
-     * Fills a moodle form with field/value data.
-     *
-     * @Given /^I fill the moodle form with:$/
-     * @throws ElementNotFoundException Thrown by behat_base::find
-     * @param TableNode $data
-     */
-    public function i_fill_the_moodle_form_with(TableNode $data) {
-
-        // Expand all fields in case we have.
-        $this->expand_all_fields();
-
-        $datahash = $data->getRowsHash();
-
-        // The action depends on the field type.
-        foreach ($datahash as $locator => $value) {
-
-            // Getting the node element pointed by the label.
-            $fieldnode = $this->find_field($locator);
-
-            // Gets the field type from a parent node.
-            $field = behat_field_manager::get_form_field($fieldnode, $this->getSession());
-
-            // Delegates to the field class.
-            $field->set_value($value);
-        }
-    }
-
     /**
      * Expands all moodleform's fields, including collapsed fieldsets and advanced fields if they are present.
      * @Given /^I expand all fieldsets$/
@@ -129,9 +99,10 @@ class behat_forms extends behat_base {
      * @return void
      */
     protected function expand_all_fields() {
-
-        // We ensure that all the editors are loaded and we can interact with them.
-        $this->ensure_editors_are_loaded();
+        // Expand only if JS mode, else not needed.
+        if (!$this->running_javascript()) {
+            return;
+        }
 
         // We already know that we waited for the DOM and the JS to be loaded, even the editor
         // so, we will use the reduced timeout as it is a common task and we should save time.
@@ -179,9 +150,6 @@ class behat_forms extends behat_base {
     /**
      * Sets the specified value to the field.
      *
-     * Backport of Moodle 2.7 method to help backporting
-     * features to 2.6.
-     *
      * @Given /^I set the field "(?P<field_string>(?:[^"]|\\")*)" to "(?P<field_value_string>(?:[^"]|\\")*)"$/
      * @throws ElementNotFoundException Thrown by behat_base::find
      * @param string $field
@@ -193,94 +161,64 @@ class behat_forms extends behat_base {
     }
 
     /**
-     * Fills in form text field with specified id|name|label|value. It works with text-based fields.
+     * Press the key in the field to trigger the javascript keypress event
      *
-     * @When /^I fill in "(?P<field_string>(?:[^"]|\\")*)" with "(?P<value_string>(?:[^"]|\\")*)"$/
+     * Note that the character key will not actually be typed in the input field
+     *
+     * @Given /^I press key "(?P<key_string>(?:[^"]|\\")*)" in the field "(?P<field_string>(?:[^"]|\\")*)"$/
+     * @throws ElementNotFoundException Thrown by behat_base::find
+     * @param string $key either char-code or character itself,
+     *          may optionally be prefixed with ctrl-, alt-, shift- or meta-
+     * @param string $field
+     * @return void
+     */
+    public function i_press_key_in_the_field($key, $field) {
+        if (!$this->running_javascript()) {
+            throw new DriverException('Key press step is not available with Javascript disabled');
+        }
+        $fld = behat_field_manager::get_form_field_from_label($field, $this);
+        $modifier = null;
+        $char = $key;
+        if (preg_match('/-/', $key)) {
+            list($modifier, $char) = preg_split('/-/', $key, 2);
+        }
+        if (is_numeric($char)) {
+            $char = (int)$char;
+        }
+        $fld->key_press($char, $modifier);
+    }
+
+    /**
+     * Sets the specified value to the field.
+     *
+     * @Given /^I set the field "(?P<field_string>(?:[^"]|\\")*)" to multiline$/
+     * @throws ElementNotFoundException Thrown by behat_base::find
+     * @param string $field
+     * @param PyStringNode $value
+     * @return void
+     */
+    public function i_set_the_field_to_multiline($field, PyStringNode $value) {
+        $this->set_field_value($field, (string)$value);
+    }
+
+    /**
+     * Sets the specified value to the field with xpath.
+     *
+     * @Given /^I set the field with xpath "(?P<fieldxpath_string>(?:[^"]|\\")*)" to "(?P<field_value_string>(?:[^"]|\\")*)"$/
      * @throws ElementNotFoundException Thrown by behat_base::find
      * @param string $field
      * @param string $value
+     * @return void
      */
-    public function fill_field($field, $value) {
-        $this->set_field_value($field, $value);
-    }
-
-    /**
-     * Selects option in select field with specified id|name|label|value.
-     *
-     * @When /^I select "(?P<option_string>(?:[^"]|\\")*)" from "(?P<select_string>(?:[^"]|\\")*)"$/
-     * @throws ElementNotFoundException Thrown by behat_base::find
-     * @param string $option
-     * @param string $select
-     */
-    public function select_option($option, $select) {
-        $this->set_field_value($select, $option);
-    }
-
-    /**
-     * Selects the specified id|name|label from the specified radio button.
-     *
-     * @When /^I select "(?P<radio_button_string>(?:[^"]|\\")*)" radio button$/
-     * @throws ElementNotFoundException Thrown by behat_base::find
-     * @param string $radio The radio button id, name or label value
-     */
-    public function select_radio($radio) {
-        $this->set_field_value($radio, 1);
-    }
-
-    /**
-     * Checks checkbox with specified id|name|label|value.
-     *
-     * @When /^I check "(?P<option_string>(?:[^"]|\\")*)"$/
-     * @throws ElementNotFoundException Thrown by behat_base::find
-     * @param string $option
-     */
-    public function check_option($option) {
-        $this->set_field_value($option, 1);
-    }
-
-    /**
-     * Unchecks checkbox with specified id|name|label|value.
-     *
-     * @When /^I uncheck "(?P<option_string>(?:[^"]|\\")*)"$/
-     * @throws ElementNotFoundException Thrown by behat_base::find
-     * @param string $option
-     */
-    public function uncheck_option($option) {
-        $this->set_field_value($option, '');
-    }
-
-    /**
-     * Checks that the field matches the specified value. When using multi-select fields use commas to separate selected options.
-     *
-     * @Then /^the "(?P<field_string>(?:[^"]|\\")*)" field should match "(?P<value_string>(?:[^"]|\\")*)" value$/
-     * @throws ExpectationException
-     * @throws ElementNotFoundException Thrown by behat_base::find
-     * @param string $locator
-     * @param string $value
-     */
-    public function the_field_should_match_value($locator, $value) {
-
-        $fieldnode = $this->find_field($locator);
-
-        // Get the field.
-        $field = behat_field_manager::get_form_field($fieldnode, $this->getSession());
-
-        // Checks if the provided value matches the current field value.
-        if (!$field->matches($value)) {
-            $fieldvalue = $field->get_value();
-            throw new ExpectationException(
-                'The \'' . $locator . '\' value is \'' . $fieldvalue . '\', \'' . $value . '\' expected' ,
-                $this->getSession()
-            );
-        }
+    public function i_set_the_field_with_xpath_to($fieldxpath, $value) {
+        $fieldNode = $this->find('xpath', $fieldxpath);
+        $field = behat_field_manager::get_form_field($fieldNode, $this->getSession());
+        $field->set_value($value);
     }
 
     /**
      * Checks, the field matches the value. More info in http://docs.moodle.org/dev/Acceptance_testing#Providing_values_to_steps.
      *
-     * Backport of Moodle 2.7 method to help backporting
-     * features to 2.6.
-     *
      * @Then /^the field "(?P<field_string>(?:[^"]|\\")*)" matches value "(?P<field_value_string>(?:[^"]|\\")*)"$/
      * @throws ElementNotFoundException Thrown by behat_base::find
      * @param string $field
@@ -289,10 +227,8 @@ class behat_forms extends behat_base {
      */
     public function the_field_matches_value($field, $value) {
 
-        $fieldnode = $this->find_field($field);
-
         // Get the field.
-        $formfield = behat_field_manager::get_form_field($fieldnode, $this->getSession());
+        $formfield = behat_field_manager::get_form_field_from_label($field, $this);
 
         // Checks if the provided value matches the current field value.
         if (!$formfield->matches($value)) {
@@ -305,9 +241,9 @@ class behat_forms extends behat_base {
     }
 
     /**
-     * Checks that the form element field does not match the specified value.
+     * Checks, the field does not match the value. More info in http://docs.moodle.org/dev/Acceptance_testing#Providing_values_to_steps.
      *
-     * @Then /^the field "(?P<field_string>(?:[^"]|\\")*)" does not match value "(?P<value_string>(?:[^"]|\\")*)"$/
+     * @Then /^the field "(?P<field_string>(?:[^"]|\\")*)" does not match value "(?P<field_value_string>(?:[^"]|\\")*)"$/
      * @throws ExpectationException
      * @throws ElementNotFoundException Thrown by behat_base::find
      * @param string $field
@@ -316,14 +252,11 @@ class behat_forms extends behat_base {
      */
     public function the_field_does_not_match_value($field, $value) {
 
-        $fieldnode = $this->find_field($field);
-
         // Get the field.
-        $formfield = behat_field_manager::get_form_field($fieldnode, $this->getSession());
+        $formfield = behat_field_manager::get_form_field_from_label($field, $this);
 
         // Checks if the provided value matches the current field value.
         if ($formfield->matches($value)) {
-            $fieldvalue = $formfield->get_value();
             throw new ExpectationException(
                 'The \'' . $field . '\' value matches \'' . $value . '\' and it should not match it' ,
                 $this->getSession()
@@ -332,7 +265,58 @@ class behat_forms extends behat_base {
     }
 
     /**
-     * Checks if fields values matches the provided values. Provide a table with field/value data.
+     * Checks, the field matches the value.
+     *
+     * @Then /^the field with xpath "(?P<xpath_string>(?:[^"]|\\")*)" matches value "(?P<field_value_string>(?:[^"]|\\")*)"$/
+     * @throws ExpectationException
+     * @throws ElementNotFoundException Thrown by behat_base::find
+     * @param string $fieldxpath
+     * @param string $value
+     * @return void
+     */
+    public function the_field_with_xpath_matches_value($fieldxpath, $value) {
+
+        // Get the field.
+        $fieldnode = $this->find('xpath', $fieldxpath);
+        $formfield = behat_field_manager::get_form_field($fieldnode, $this->getSession());
+
+        // Checks if the provided value matches the current field value.
+        if (!$formfield->matches($value)) {
+            $fieldvalue = $formfield->get_value();
+            throw new ExpectationException(
+                'The \'' . $fieldxpath . '\' value is \'' . $fieldvalue . '\', \'' . $value . '\' expected' ,
+                $this->getSession()
+            );
+        }
+    }
+
+    /**
+     * Checks, the field does not match the value.
+     *
+     * @Then /^the field with xpath "(?P<xpath_string>(?:[^"]|\\")*)" does not match value "(?P<field_value_string>(?:[^"]|\\")*)"$/
+     * @throws ExpectationException
+     * @throws ElementNotFoundException Thrown by behat_base::find
+     * @param string $fieldxpath
+     * @param string $value
+     * @return void
+     */
+    public function the_field_with_xpath_does_not_match_value($fieldxpath, $value) {
+
+        // Get the field.
+        $fieldnode = $this->find('xpath', $fieldxpath);
+        $formfield = behat_field_manager::get_form_field($fieldnode, $this->getSession());
+
+        // Checks if the provided value matches the current field value.
+        if ($formfield->matches($value)) {
+            throw new ExpectationException(
+                'The \'' . $fieldxpath . '\' value matches \'' . $value . '\' and it should not match it' ,
+                $this->getSession()
+            );
+        }
+    }
+
+    /**
+     * Checks, the provided field/value matches. More info in http://docs.moodle.org/dev/Acceptance_testing#Providing_values_to_steps.
      *
      * @Then /^the following fields match these values:$/
      * @throws ExpectationException
@@ -347,12 +331,12 @@ class behat_forms extends behat_base {
 
         // The action depends on the field type.
         foreach ($datahash as $locator => $value) {
-            $this->the_field_should_match_value($locator, $value);
+            $this->the_field_matches_value($locator, $value);
         }
     }
 
     /**
-     * Checks that fields values do not match the provided values. Provide a table with field/value data.
+     * Checks that the provided field/value pairs don't match. More info in http://docs.moodle.org/dev/Acceptance_testing#Providing_values_to_steps.
      *
      * @Then /^the following fields do not match these values:$/
      * @throws ExpectationException
@@ -371,26 +355,6 @@ class behat_forms extends behat_base {
         }
     }
 
-    /**
-     * Checks, that checkbox with specified in|name|label|value is checked.
-     *
-     * @Then /^the "(?P<checkbox_string>(?:[^"]|\\")*)" checkbox should be checked$/
-     * @param string $checkbox
-     */
-    public function assert_checkbox_checked($checkbox) {
-        $this->the_field_should_match_value($checkbox, 1);
-    }
-
-    /**
-     * Checks, that checkbox with specified in|name|label|value is unchecked.
-     *
-     * @Then /^the "(?P<checkbox_string>(?:[^"]|\\")*)" checkbox should not be checked$/
-     * @param string $checkbox
-     */
-    public function assert_checkbox_not_checked($checkbox) {
-        $this->the_field_should_match_value($checkbox, '');
-    }
-
     /**
      * Checks, that given select box contains the specified option.
      *
@@ -491,12 +455,37 @@ class behat_forms extends behat_base {
      */
     protected function set_field_value($fieldlocator, $value) {
 
-        $node = $this->find_field($fieldlocator);
-
         // We delegate to behat_form_field class, it will
         // guess the type properly as it is a select tag.
-        $field = behat_field_manager::get_form_field($node, $this->getSession());
+        $field = behat_field_manager::get_form_field_from_label($fieldlocator, $this);
         $field->set_value($value);
     }
 
+    /**
+     * Select a value from single select and redirect.
+     *
+     * @Given /^I select "(?P<singleselect_option_string>(?:[^"]|\\")*)" from the "(?P<singleselect_name_string>(?:[^"]|\\")*)" singleselect$/
+     */
+    public function i_select_from_the_singleselect($option, $singleselect) {
+        $actions = array(
+            new Given('I set the field "' . $this->escape($singleselect) . '" to "' . $this->escape($option) . '"'),
+        );
+
+        if (!$this->running_javascript()) {
+            // Press button in the specified select container.
+            $containerxpath = "//div[" .
+                "(contains(concat(' ', normalize-space(@class), ' '), ' singleselect ') " .
+                    "or contains(concat(' ', normalize-space(@class), ' '), ' urlselect ')".
+                ") and (
+                .//label[contains(normalize-space(string(.)), '" . $singleselect . "')] " .
+                    "or .//select[(./@name='" . $singleselect . "' or ./@id='". $singleselect . "')]" .
+                ")]";
+
+            $actions[] = new Given('I click on "' . get_string('go') . '" "button" in the "' . $containerxpath .
+                '" "xpath_element"');
+        }
+
+        return $actions;
+    }
+
 }
