@@ -28,18 +28,49 @@ Y.extend(CONFIRM, M.core.notification.info, {
      * @private
      */
     _closeEvents: null,
+
+    /**
+     * A reference to the yes button.
+     *
+     * @property _yesButton
+     * @type Node
+     * @private
+     */
+    _yesButton: null,
+
+    /**
+     * A reference to the No button.
+     *
+     * @property _noButton
+     * @type Node
+     * @private
+     */
+    _noButton: null,
+
+    /**
+     * A reference to the Question.
+     *
+     * @property _question
+     * @type Node
+     * @private
+     */
+    _question: null,
+
     initializer: function() {
         this._closeEvents = [];
         this.publish('complete');
         this.publish('complete-yes');
         this.publish('complete-no');
-        var yes = Y.Node.create('<input type="button" id="id_yuiconfirmyes-' + this.get('COUNT') + '" value="'+this.get(CONFIRMYES)+'" />'),
-            no = Y.Node.create('<input type="button" id="id_yuiconfirmno-' + this.get('COUNT') + '" value="'+this.get(CONFIRMNO)+'" />'),
-            content = Y.Node.create('<div class="confirmation-dialogue"></div>')
-                        .append(Y.Node.create('<div class="confirmation-message">'+this.get(QUESTION)+'</div>'))
+        this._yesButton = Y.Node.create('<input type="button" id="id_yuiconfirmyes-' +
+                                        this.get('COUNT') + '" value="'+this.get(CONFIRMYES)+'" />');
+        this._noButton = Y.Node.create('<input type="button" id="id_yuiconfirmno-' +
+                                        this.get('COUNT') + '" value="'+this.get(CONFIRMNO)+'" />');
+        this._question = Y.Node.create('<div class="confirmation-message">' + this.get(QUESTION) + '</div>');
+        var content = Y.Node.create('<div class="confirmation-dialogue"></div>')
+                        .append(this._question)
                         .append(Y.Node.create('<div class="confirmation-buttons"></div>')
-                            .append(yes)
-                            .append(no));
+                            .append(this._yesButton)
+                            .append(this._noButton));
         this.get(BASE).addClass('moodle-dialogue-confirm');
         this.setStdModContent(Y.WidgetStdMod.BODY, content, Y.WidgetStdMod.REPLACE);
         this.setStdModContent(Y.WidgetStdMod.HEADER,
@@ -47,8 +78,8 @@ Y.extend(CONFIRM, M.core.notification.info, {
 
         this._closeEvents.push(
             Y.on('key', this.submit, window, 'down:27', this, false),
-            yes.on('click', this.submit, this, true),
-            no.on('click', this.submit, this, false)
+            this._yesButton.on('click', this.submit, this, true),
+            this._noButton.on('click', this.submit, this, false)
         );
 
         var closeButton = this.get('boundingBox').one('.closebutton');
@@ -84,7 +115,15 @@ Y.extend(CONFIRM, M.core.notification.info, {
          */
         yesLabel: {
             validator: Y.Lang.isString,
-            value: M.util.get_string('yes', 'moodle')
+            valueFn: function() {
+                return M.util.get_string('yes', 'moodle');
+            },
+            setter: function(value) {
+                if (this._yesButton) {
+                    this._yesButton.set('value', value);
+                }
+                return value;
+            }
         },
 
         /**
@@ -96,7 +135,15 @@ Y.extend(CONFIRM, M.core.notification.info, {
          */
         noLabel: {
             validator: Y.Lang.isString,
-            value: M.util.get_string('no', 'moodle')
+            valueFn: function() {
+                return M.util.get_string('no', 'moodle');
+            },
+            setter: function(value) {
+                if (this._noButton) {
+                    this._noButton.set('value', value);
+                }
+                return value;
+            }
         },
 
         /**
@@ -120,7 +167,15 @@ Y.extend(CONFIRM, M.core.notification.info, {
          */
         question: {
             validator: Y.Lang.isString,
-            value: M.util.get_string('areyousure', 'moodle')
+            valueFn: function() {
+                return M.util.get_string('areyousure', 'moodle');
+            },
+            setter: function(value) {
+                if (this._question) {
+                    this._question.set('value', value);
+                }
+                return value;
+            }
         }
     }
 });
